
\def\ttt{2--3 tree}
\def\ttts{\ttt s}

@* Introduction.

We're going to put together a library like the standard \UNIX\ {\sl
tsearch\/} library, only ours will use \ttts\ instead of binary trees.
\ttts\ are like binary trees, but they're more efficient to search; they
don't have the awful worst-case behavior that binary trees have.

The idea behind a \ttt\ is this: Each node of a \ttt\ contains either
one or two keys, and has respectively two or three children (of which
all or none are |NULL|.)

If there is one key in a node, then the left subtree of that node
contains only keys that are lexicographically before the key in that
node, and the right subtree contains only keys that are
lexicographically after the key in that node, just as in a regular
binary tree.

If there are two keys in a node, then the keys in the tree obey the
order relation $$ keys~in~left~subtree < first~key <
keys~in~middle~subtree < second~key < keys~in~right~subtree$$ Where $<$
here means `is lexicographically before'.

We'll have several functions for using the \ttts.  The interface will be
through an abstract `key' type, which will be passed around as a |void
*|.  The user will supply a function which compares two keys and returns
a result \` a la |strcmp|.  Since the comparison function need not
compare all the data contained in the `keys', the keys can contain the
keyed data as well as the comparison keys.   Note that in particular
this implies that two nonidentical keys may `match'.

@ The interface presented by the \UNIX\ {\sl tsearch\/} library isn't
quite what we want; it returns a pointer to the node in which the key
was found.  Since the nodes returned by {\sl tsearch\/} contain only one
key each, it's reasonable to expect the user to extract them.  But our
nodes will contain up to two keys each, and it's sill to expect the user
to do compares and whatnot to find the key he or she wants after we've
already done that.  So we'll return a pointer to the key we found,
instead, and have our routines take extra parameters that say whether
to replace new keys with old and so on.  We'll call this general-purpose
search/replace/add routine |t23search|.

So the outline of our little library looks like this:

@c
@<includes@> @/
@<defines@> @/
@<types@> @/
@<prototypes@> @/
@#
@<|t23search| function@> @/
@<|t23first| function @> @/
@<|t23next| function @> @/
@#
@<auxiliary functions@>

@ To begin with we'll need |stdio.h| for our error messages.

@<includes@> =

#include <stdio.h>

@* Basic data types.

There are two important data types we'll need to communicate to the
end-user:  We'll need to establish a convention for keys, and we'll need
to establish a convention for \ttts\ themselves.  This latter is so that
the user can use our routines to manage several trees at once; each of
our programs will accept a parameter which is the root of the tree on
which to operate.

Keys will be represented as |void *|, because the user can define an
arbitrary structure and pass us a |void *|-cast pointer to it:

@<types@> =

typedef void *T23_KEY;

@ \ttts and \ttt\ nodes will be represented by the same data type, since
there's such an obvious correspondence between them: Any \ttt\ can be
identified with the single node at its root, and any \ttt\ node can be
identified with the entire \ttt\ of which it is the root.  A \ttt\ has
up to two keys and up to three children, so we'll have fields in or
\ttt-node structure for those.

To simplify bookkeeping later on we'll also keep track of the number of
keys in each node and of the parent of each node.  We could determine
the number of keys in a node by looking at the second key and
establishing the convention that when there's only one key, the second
one is |NULL|, but having a |numkeys| field is conceptually simpler.

@<types@> =

typedef struct t23_st_node {
int numkeys;
T23_KEY key[2];
struct t23_st_node *child[3];
struct t23_st_node *parent;
} *T23;

@ We'll also expect the user to be passing us a pointer to a function
which compares keys, and we'll have to deal with such pointers fairly
often, so let's declare a type for these functions:  They take two
arguments of type |KEY| and yield an |int| result.

@<types@> =

typedef int (*T23_COMPF)(T23_KEY, T23_KEY);

@ We need to think about the interface to |t23search|.  We'll give it
the usual parameters: key to search for, tree to search in, comparison
function to use; we'll also give it two extra parameters: What to do if
the search is successful (replace old key with new, or leave the old key
alone?), and what to do if the search is unsuccessful (insert the new
key, or return |NULL|?)

After a successful search, |t23search| will return the key it found.  If
this key was replaced by a new key, the user will want to old one back
anyway, so that they can free the memory in it or otherise handle it,
and presumably they still know how to find the key that replaced it.
After an unsuccessful search, |t23search| can simply return its original
argument; this makes lots of sense when that argument was inserted into
the tree, and it is a simple and consistent way to indicate failure even
in the case where the new key wasn't to be inserted into the tree.
This method also allows us to reseve a |NULL| return for those times
when a lack of memory or some other error prevents |t23search| from
doing its job.

The declaration for |t23search| looks like this:

@<|t23search| declaration@> = 

T23_KEY t23search(T23_KEY key, T23 *rootp, T23_COMPF compf, 
t23_boolean insert_on_success, t23_boolean insert_on_failure)

@ @<prototypes@> = 

@<|t23search| declaration@> ;

@ Note in particular that we pass in a {\sl pointer\/} to the root of
the tree to search, rather than the root itself.  That's so that
|t23search| can modify the calling procedure's idea of where the root is
if it needs t.  In particular, the calling procedure can pass in a null
pointer as the root, indicating that it wants |t23search| to start a new
tree; |t23search| can create a new tree and communicate back to the
calling process where it is.

We also specify two |t23_boolean| arguments which describe whether or
not to insert the new key at the termination of a successful search (in
which case the new key replaces the old one) or an unsuccessful search
(in which case the new key is inserted at an appropriate place.)  Now we
need a boolean type.  We'll take the traditional option:

@<types@> =

typedef enum t23_en_bool {T23_FALSE = 0, T23_TRUE = 1} t23_boolean;

@* |t23search|.

Here we'll begin the |t23search| routine itself.  There are two major
kinds of action we need to perform when we're building a \ttt.  First,
we search down the tree, in a manner analagous to the way we search a
binary tree, until we either find what we're looking for, or reach the
leaves.  This will form the |t23search| half of our
search-and-maybe-insert routine.  If we need to insert a new key into
the tree, there might be room for it in the appropriate node (that is,
The node is already there and has only one key) in which case our job is
easy, or we might have to split the node in two and percolate up.  In
this case we'll call a subsidiary routine, |t23_up|, which may or may
not keep going up, up, up, until finally it splits the root.

The outline for |t23search| thus looks like this:

@<|t23search| function@> =

@<|t23search| declaration@> { @/
@#
@<|t23search| local variables@> @;
@#
@<handle null tree case@> @;
@#
for (;;) { @/
@<compare |key| to keys in current node@> @;
@#
@<if there's a match@> {
@<terminate, search successful@> @;
} else if (@<we're at a leaf@>) {
@<terminate, search failed@> @;
} else {
@<move down@> @;
}
}
}


@ It'll be convenient to define a function which takes care of the
details of comparing a new key to the keys in a certain node, because
we'll be doing that all the time.  We'll call it |t23_aux_compare|.

@<auxiliary functions@> =

@<|t23_aux_compare| function@>

@ There are five possibilities for where a given key can lie with
respect to a given node:  It can match either of the two keys that are
already in the node, or it can lie before (left of) both, after (right
of) both, or after the first and before the second (in between).

@<defines@> = 

#define POS_LEFT 000
#define POS_MATCH_FIRST 010
#define POS_MIDDLE 001
#define POS_MATCH_SECOND 020
#define POS_RIGHT 002

@ Note that if there was no match (that is, we got one of |POS_LEFT|,
|POS_MIDDLE|, or |POS_RIGHT|), the value we got is the index number of
the subtree we have next to search; this makes things easier later on.

We'll also define some macros to make processing these codes easier: we
can take the value returned by our comparison function and run it
through |MATCHED|; we get 0 if the new key didn't match either key
in the node, nonzero otherwise.  If there was a match, |WHICH_MATCHED|
will tell us the index number of the key that was matched.

@<defines@> =

#define MATCHED(code) ((code) & 070)
#define WHICH_MATCHED(code) (((code) >> 3) - 1)

@ Our comparison function will take a key and a node and return one of
the integer codes above.  We'll call it |t23_aux_compare|.

@<|t23_aux_compare| declaration@> =

int t23_aux_compare(T23_KEY key, T23 node, T23_COMPF compf)

@ @<prototypes@> =

@<|t23_aux_compare| declaration@> ;

@ @<|t23_aux_compare| function@> =

@<|t23_aux_compare| declaration@> { @/
@#
int compresult;
@#
compresult = (*compf)(key, node->key[0]);
@#
if (compresult < 0) return POS_LEFT;
else if (compresult == 0) return POS_MATCH_FIRST;
else if (node->numkeys == 1) return POS_MIDDLE;
@#
compresult = (*compf)(key, node->key[1]);
@#
if (compresult < 0) return POS_MIDDLE;
else if (compresult == 0) return POS_MATCH_SECOND;
else return POS_RIGHT;
}


@ We use |t23_aux_compare| again to find where in or below |cur| |key|
belongs, and we'll reuse this value several times, so it's best to save
it in a variable called |direction|.

@<compare |key| to keys...@> =

direction = t23_aux_compare(key, cur, compf);

@ Here |cur| represents `where we are' in the process of tracing down
the tree; we start at the root and work down.

@<|t23search| local variables@> =

T23 cur = *rootp;
int direction;

@ Given our macros, it's easy to see if there was a match or not:

@<if there's a match@> =

if (MATCHED(direction))

@ If we did match, then the search was successful.  We either replace
the old key with the new, or we do nothing, depending on the setting of
the |insert_on_success| parameter.  In either case we return the key
that actually winds up in the \ttt.

@<terminate, search successful@> =

if ( insert_on_success ) cur->key[WHICH_MATCHED(direction)] = key;
return cur->key[WHICH_MATCHED(direction)];

@ Otherwise, the search wasn't successful.  If we're not at a leaf node,
we can keep going.  

@<we're at a leaf@> =

(isleaf(cur))

@ @<defines@> =

#define isleaf(node)  (node->child[0] == NULL)

@ @<move down@> =

cur = cur->child[direction];

@ On the other hand, if we {\sl are\/} at a leaf, we have nowhere to go,
and so our search failed.  We either call |t23_up| to insert for us, or
we return a null node, depending on the setting of the
|insert_on_failure| parameter.  |t23_up| will return the usual nonzero
failure code.

@<terminate, search failed@> =

if (insert_on_failure) {
  T23 newnode = makenewnode(key);

  if (newnode != NULL && t23_up(newnode, cur, rootp, compf) == 0)
    return key;
  else
    return NULL;
}
return key;

@ Before we go to think about |t23_up|, we need to handle one special
case:  What if the tree we were asked to search was empty?  In that case
our search automatically fails, so the code looks a lot like what we
just did.

@<handle null tree case@> =

if (*rootp == NULL) {
if (insert_on_failure) {
   T23 newnode = makenewnode(key);
   if (newnode == NULL) return NULL;
   else *rootp = newnode;
}
return key;
}


@* |t23_up|.

|t23_up| is the routine that handles the difficult details of inserting
a new key into the tree.  There are two basic cases it has to deal with.
In one case, |t23_up| is asked to insert a new key into a node that has
only one key; this is easy.  In the other case, it has to insert the new
key into a node which is already full (that is, a node with two keys).
In this second case it must split the full node, move keys around, and
try to insert the middle of the three keys into the parent node.  If
there is no parent node, a new root must be built.

@<auxiliary functions@> = 

@<|t23_up| function@>

@ |t23_up| has almost the same parameters as |t23_search|: it takes node
to insert into, a pointer to the root node, and a comparison function.
However, instead of taking a raw key, it takes a new node which contains
that key; that way the node to be inserted can actually be the root of a
nontrivial tree.  That is because |t23_up| will construct a tree of
split nodes as it works its way up the tree, and may have to insert that
entire tree in as a subtree of some node.

|t23_up| will have a rudimentary return value, since the only reason we
call it is for its side effect of inserting a new key into the tree.  It
will report back an integer error status, which will be zero if all went
well and nonzero if something disastrous happened.  (Typically, if a
call to |makenewnode| found no more memory available.)

@<|t23_up| declaration@> =

static int t23_up(T23 newnode, T23 oldnode, T23 *rootp, T23_COMPF compf)

@ @<prototypes@> =

@<|t23_up| declaration@> ;

@ @<|t23_up| function@> =

@<|t23_up| declaration@> { @/
@#
@<|t23_up| local variables@> @;
@#
for (;;) { @/
@<if |oldnode| is not full@> {
@<do easy insert@> @;
return 0;
} else { @/
@<build new nodes@> @;
@<link together new nodes@> @;
@<prepare to insert new tree into parent@> @;
}
}
}

@ Testing to see if |oldnode| is full or not is made easier by the
presence of the |numkeys| field in each node.

@<if |oldnode| is not full@> =

if (oldnode->numkeys == 1)

@ Even the `easy insert' is surprisingly complicated.  The |oldnode| has
two children, one of whom is actually the |newnode|, and we have to
decide which one; the |newnode| has two children also, and this makes
three children in all (because the |newnode| doesn't count as a child
any more.)  We may have to shuffle the key in |oldnode| around in
|oldnode| to put it in the right place.  We'll need the key from
|newnode| here, and it turns out we'll need it later too, so for
efficiency let's get it out once at the beginning of |t23_up|.

@<|t23_up| local variables@> =

T23_KEY newkey = newnode->key[0];

@ @<do easy insert@> =

int position = t23_aux_compare(newkey, oldnode, compf);
int i;
@#
switch(position) {
case POS_LEFT: @/
oldnode->key[1] = oldnode->key[0];
oldnode->child[2] = oldnode->child[1];
@#
oldnode->child[0] = newnode->child[0];
oldnode->key[0] = newkey;
oldnode->child[1] = newnode->child[1];
break;
@#
case POS_MIDDLE: @/
oldnode->child[1] = newnode->child[0];
oldnode->key[1]   = newkey;
oldnode->child[2] = newnode->child[1];
break;
default: /* This can't happen */
fprintf(stderr, "T23 package internal logic bug #7\n");
return -1;
}
@#
oldnode->numkeys = 2;
@#
/* Fix |parent| fields in children, who may have moved. */
for (i=0; i<3; i++) {
if(oldnode->child[i])
oldnode->child[i]->parent = oldnode;
}
@#
/* We've inserted the new key into a node now, so we don't need the
 node that used to contain it any more. 
*/
free(newnode);

@* Splitting.

Now we get to the hard part of the program:  We're trying to insert a
key into a node for which there's no room.  

There are three cases that have to be handled separately, depending on
whether |newnode| lay to the left of, in between, or to the right of the
two keys in |oldnode|.  For symmetry we'll build afresh the three new
nodes |leftnode|, |middlenode|, and |rightnode|, copy the appropriate
keys into them, and then throw away |oldnode| and |newnode|.

@<build new nodes@> =

int position = t23_aux_compare(newkey, oldnode, compf);
T23 leftnode, middlenode, rightnode;
@#
switch(position) {
case POS_LEFT: @/
leftnode = makenewnode(newkey);
middlenode = makenewnode(oldnode->key[0]);
rightnode = makenewnode(oldnode->key[1]);
break;
case POS_MIDDLE: @/
leftnode = makenewnode(oldnode->key[0]);
middlenode = makenewnode(newkey);
rightnode = makenewnode(oldnode->key[1]);
break;
case POS_RIGHT: @/
leftnode = makenewnode(oldnode->key[0]);
middlenode = makenewnode(oldnode->key[1]);
rightnode = makenewnode(newkey);
break;
default: /* This can't happen */
fprintf(stderr, "T23 package internal logic bug #8\n");
return -1;
}

if (leftnode == NULL || middlenode == NULL || rightnode == NULL)
  return -1; /* Out of memory */


@ Linking the nodes together into a little tree is just a matter of
juggling pointers appropriately.

@<link together new nodes@> =

/* The next four lines correspond to part ofthe high-level task of
``promoting the middle key into the parent node''.  The other half will
come on the next iteration through the |t23_up| loop, when we merge
|middlenode| with the real parent.
*/

middlenode->child[0] = leftnode;
middlenode->child[1] = rightnode;
leftnode->parent = middlenode;
rightnode->parent = middlenode;
@#
switch(position) {
case POS_LEFT: @/
leftnode->child[0] = newnode->child[0];
leftnode->child[1] = newnode->child[1];
rightnode->child[0] = oldnode->child[1];
rightnode->child[1] = oldnode->child[2];
break;
case POS_MIDDLE: @/
leftnode->child[0] = oldnode->child[0];
leftnode->child[1] = newnode->child[0];
rightnode->child[0] = newnode->child[1];
rightnode->child[1] = oldnode->child[2];
break;
case POS_RIGHT: @/
leftnode->child[0] = oldnode->child[0];
leftnode->child[1] = oldnode->child[1];
rightnode->child[0] = newnode->child[0];
rightnode->child[1] = newnode->child[1];
break;
default: /* This can't happen */
fprintf(stderr, "T23 package internal logic bug #9\n");
return -1;
}
@#
/* Fix |parent| fields in children, who may have moved. */
if (leftnode->child[0]) leftnode->child[0]->parent = leftnode;
if (leftnode->child[1]) leftnode->child[1]->parent = leftnode;
if (rightnode->child[0]) rightnode->child[0]->parent = rightnode;
if (rightnode->child[1]) rightnode->child[1]->parent = rightnode;

@ Now we've constructed a little subtree which has the node that we
wanted to split all split, and it has the middle key from the split node
all set in a new node, namely |middlenode|.  We are ready to move up the
tree.  If we're at the root already, then we're done---|middlenode| is
the new root.  Otherwise, insert |middlenode| into |oldnode|'s parent.

@<prepare to insert new tree into parent@> =

if (oldnode->parent == NULL) {
*rootp = middlenode;
free(oldnode);
free(newnode);
return 0; /* Success! */
} else {
T23 old_oldnode = oldnode;
@#
oldnode = oldnode->parent;
free(old_oldnode);
free(newnode);
newnode = middlenode;
newkey = newnode->key[0];
/* Fall through and iterate again. */
}


@* |t23first|.

Now that we have a routine to build and search trees, we need one to
dump them out again.  

First let's look at the \UNIX\ |tsearch| library again, and see why we
don't want to do it that way.

The |tsearch| routine, |twalk|, traces a binary tree, and, at each node,
calls a user-supplied function which takes three parameters.  The first
parameter is a pointer to the current node.  The second parameter is a
member of an enum type which describes whether the node is beging
visited for the first time (its left subtree has not yet been visited),
the second time (its left subtree has been visited but its right subtree
has not), the third time (both subtrees have been visited), or whether
it is a leaf node (leaves get visited only once.)  The third parameter
is the distance of the node from the root.

That way you can invoke your own function on each node in the tree in
preorder, inorder, or postorder.

Unfortunately, this is not a useful functionality to have.  `preorder,'
`inorder,' and `postorder' only make sense in the context of a binary
tree.  However, the rest of |tsearch|'s user interface is opaque and the
binary tree itself is completely invisible.  `preorder' might be useful,
for example, if one were using |tsearch| to manage an expression tree,
but the insertion and deleteion primitives don't allow they; they are
for managing a sorted binary tree, and not an expression tree.

Once you realize that |tsearch| and |t23search| are simply opaque
ordered-database managers, it becomes clear that the correct interface
for an examination or dump of all the keys they contain is the one
presented by the \UNIX\ |DBM| package: There should be a function that
fetches the (lexicographically) first key from the database, and then a
function which, given a key, gets the next key.

The first of these, |t23first|, is simpler, so we'll tackle it first.
Note that |t23first| only needs to know the root of the tree we're
interested in; the comparition function is unnecessary because the
comparision function information is embedded in the structure of the
tree itself.

@<|t23first| declaration@> =

T23_KEY t23first(T23 root)

@ @<prototypes@> =

@<|t23first| declaration@> ;

@ |t23first| is easy to write:  We just go left until we can't go left
any more, and then the leftmost key in the node we've come to is the key
that is lexicographically first.

@<|t23first| function@> = 

@<|t23first| declaration@> 
{
  T23 leftmostnode;

  leftmostnode = leftmost_child(root);

  return leftmostnode->key[0];
}

@ |leftmost_child| is a local function which, given a \ttt\ node, finds
the leftmost child node of the given node.

@* |t23next|.

|t23next| will take a key $k$ and a root node and will return the key
$n$ in the tree such that both $n>k$ and $\forall m: m<k \hbox{ or }
n<m$. 

There is one big difficulty.  We only want to pass keys out of the |t23|
package, and never whole tree nodes, on the principle that the data
structure we're using to track the data should be invisible to the user.
Hence, |t23nxt| gets handed a |T23_KEY| and is told to find the next key
in the tree.  This would be easy if we knew where the given key was in
the tree, but we don't; we'll have to go looking for it, and that means
searching the entire tree.  

Unfortunately, the user must pass in the comparision function, so that
if we need to find the given key in the tree again, we can do it
quickly.  An improvement to this routine would be to allow a NULL
comparision function, and to have |t23next| do a slow linear search to
find the given key if it had to.

|t23next| will return the |T23_KEY| that follows its argument in
lexicographic order, or, in the case that the key that was passed in was
lexicographically last, |NULL|.

@<|t23next| declaration@> = 

T23_KEY t23next(T23_KEY key, T23 root, T23_COMPF compf)

@ @<prototypes@> =

@<|t23next| declaration@> ;

@ @<|t23next| function@> =
@<|t23next| declaration@> @/
{ @/
  @<|t23next| local variables@>
@#
    oldnode = t23search(key, &root, compf, T23_FALSE, T23_FALSE);
@#
  position = WHICH_MATCHED(t23_aux_compare(key,oldnode,compf));
@#
  @<find and return next key@> @;
}

@ The meat of finding the next key in the \ttt\ is very complicated;
there are a lot of different cases that can arise.  First, if |key| is a
left key and the middle subtree exists, or if |key| is a right key and
the right subtree exists, then the key we want is the leftmost key in
the appropriate subtree.

@<find and return next key@> =

  if (oldnode->child[position+1]) 
    return                leftmost_child(oldnode->child[position+1])->key[0]; 

@ Otherwise, if |key| is a left key, the next key is the right key in
the same node, if there is one.

@<find and return next key@> = 

  else if (position == 0 && oldnode->numkeys == 2) @/
     return oldnode->key[1];

@ Otherwise, we have to go up, because the `next' key does not lie in
the current subtree.  If we can't go up, it's because |key| was
lexicographically last, so we need to halt.  

After going up one node, we'll try to take the next key to the right:
If we finished the left subtree of the parent node, we'll take the left
key; if we finished tracing the middle subtree, we'll take the right key
if it exists.  Otherwise the key we want isn't in the expanded subtree
either, and we have to keep going up.

@<find and return next key@> = 
@/
  /* else go up */

  while (oldnode->parent != NULL) { @/
@#
    position = t23_aux_compare(oldnode->key[0], oldnode->parent, compf); 
    oldnode = oldnode->parent;
@#
    switch(position) { @/
    case POS_LEFT:
      return oldnode->key[0]; @/
    case POS_MIDDLE: @/
      if (oldnode->numkeys == 2) @/
        return oldnode->key[1];
      /* else keep going up */
      else break;
    case POS_RIGHT:
      /* Keep going up */
      break; @/
    default:  /* This can't happen */
      fprintf(stderr, "T23 package internal logic bug #72\n");	
      return NULL;
    } @/
  } @/
@#
  /* If we're here, we reached the root of the tree going up, and
there's nowhere left to go, so the key we got must have been the last
one, so return NULL. */
@#
  return NULL;

@ We've used the variables |oldnode|, which is the node in which |key|
is stored, and |position|, which is the position of |key| within
|oldnode|, so we'd better declare them.

@<|t23next| local variables@> =

T23 oldnode;
int position;

@* Last licks.

There are just a few things we left undone.  We never defined the
function |makenewnode|, which allocates and initializes a new node with
a given key.  We never defined the function |leftmost_child|, which
finds the leftmost subnode of a given node.

@<|makenewnode| declaration@> =

static T23 makenewnode(T23_KEY key)

@ @<prototypes@> = 

@<|makenewnode| declaration@> ;

@ @<auxiliary functions@> =

@<|makenewnode| declaration@> { @/
@#
T23 temp;
@#
temp = (T23) (malloc(sizeof(struct t23_st_node)));
@#
temp->parent = NULL; 
@#
temp->child[0] = NULL;
temp->child[1] = NULL;
temp->child[2] = NULL; 
@#
temp->key[0] = key;
temp->key[1] = NULL;
@#
temp->numkeys = 1;
@#
return temp;
}

@ |makenewnode| uses |malloc|, so we have to include the declaration for
it:

@<includes@> =

#include <malloc.h>

@ |leftmost_child| is easy:

@<|leftmost_child| declaration@> =

static T23 leftmost_child(T23 node)

@ @<prototypes@> = 

  @<|leftmost_child| declaration@> ;

@ @<auxiliary functions@> =

@<|leftmost_child| declaration@>
{
  if (node->child[0]) return leftmost_child(node->child[0]);
  else return node;
}

@ Last of all we'll need an include file for user programs to include,
which will define the package's data types and prototype its functions.
We'll call this file |23.h|.

@(23.h@> =

@<includes@> @/
@<defines@> @/
@<types@> @/
@<prototypes@> @/

@ We are now finished.
\vfill \eject
